        PAGE	60,101
	TITLE	EAGLE 8530 DRIVERS
	NAME	EAGLE
;..........................................................
; SYNCHRONOUS DEVICE DRIVERS FOR THE 8530 ON THE PC BUS
; INTERRUPT PROCESSING
;
; STARTUP ESTABLISHING SEGMENTS
;   FIRST, THE STACK SEGMENT
;..........................................................
STACK	SEGMENT	PARA STACK 'STACK'
	DB	1024 DUP (0FFH)	;1K STACK SPACE
STACK	ENDS
;
;  NOW THE DATA DEFINITIONS
;
DATA	SEGMENT	PARA PUBLIC 'DATA'
;
; DOS EQUATES
;
PRT_STR	EQU	09H	;DOS PRINT-STRING FUNCTION
DOSINT	EQU	21H	;DOS CALL INTERRUPT
;
; **************************************************************
;  			8530 SCC definitions			
; **************************************************************

SCCNULL		EQU	0H	;  a null command for the SCC 
;
; 		EAGLE 8530 SCC Address Locations			
;
SCCB_CTL	EQU	300H  ;  SCC channel B control port  (J2) 
SCCB_DATA	EQU	301H  ;  SCC Channel B data port (J2)
SCCA_CTL	EQU	302H  ;  SCC Channel A control port (J1)
SCCA_DATA	EQU	303H  ;  SCC Channel A data port  (J1)
;
ECTRL_REG	EQU	304H  ; CONTROL REGISTER
;				BIT 0 - DMA CTL/DATA
;					0=DATA PORT
;					1=CTL PORT
;				BIT 1 - DMA CHANNEL
;					0=A
;					1=B
;				BIT 2 - DMA GATE
;					0=DMA DISABLE
;					1=DMA ENABLED
;				BIT 3 - INT GATE
;					0=INT DISABLED
;					1=INT ENABLED
;				BIT 4 - INT ACK
;					0=ACK TOGGLE
;					1=ACK TOGGLE 				
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; CHANNEL A
;	J1 PIN 1 - DTR OUT
;	J1 PIN 2 - RXD IN
;	J1 PIN 3 - TXD OUT
;	J1 PIN 4 - RTS OUT
;	J1 PIN 5 - CTS IN
;	J1 PIN 6 - DCD IN
;	J1 PIN 7 - TRXC IN
;	J1 PIN 8 - RTXC IN
; CHANNEL B
;	J2 PIN 2 - RXD IN   ;NOTE THIS IS TOTALLY BOGUS-SHOULD BE TXD OUT
;	J2 PIN 3 - TXD OUT  ;NOTE THIS IS TOTALLY BOGUS-SHOULD BE RXD IN
;	J2 PIN 4 - RTS OUT
;	J2 PIN 5 - CTS IN
;	J2 PIN 6 - DCD IN
;	J2 PIN 7 - GROUND
;	J2 PIN 15 - RTXC IN
;	J2 PIN 18 - SYNC OUT
;	J2 PIN 20 - DTR OUT
; **************************************************************
; 			SCC Registers				
; **************************************************************

SCCREG0		EQU	0	;  SCC register 0 
SCCREG1		EQU	1	;  SCC register 1 
SCCREG2		EQU	2	;  SCC register 2 
SCCREG3		EQU	3	;  SCC register 3 
SCCREG4		EQU	4	;  SCC register 4 
SCCREG5		EQU	5	;  SCC register 5 
SCCREG6		EQU	6	;  SCC register 6 
SCCREG7		EQU	7	;  SCC register 7 
SCCREG8		EQU	8	;  SCC register 8 
SCCREG9		EQU	9	;  SCC register 9 
SCCREG10	EQU	10	;  SCC register 10 
SCCREG11	EQU	11	;  SCC register 11 
SCCREG12	EQU	12	;  SCC register 12 
SCCREG13	EQU	13	;  SCC register 13 
SCCREG14	EQU	14	;  SCC register 14 
SCCREG15	EQU	15	;  SCC register 15 


; **************************************************************
; 		SCC Read Registers				
; **************************************************************

; 	Read Register 0  Tx/Rx and External Flag masks 

RX_ABORT	EQU	80H	;  Receive break/abort 
TX_UNRUN	EQU	40H	;  Transmit underrun 
CTS_PIN		EQU	20H	;  SCC CTS pin 
RX_HUNT		EQU	10H	;  sync/hunt flag 
DCD_PIN		EQU	08H	;  SCC DCD pin 
TX_TBE		EQU	04H	;  Transmit buffer empty 
ZERO_CNT	EQU	02H	;  Baud-rate gen. zero count 
RX_RDA		EQU	01H	;  Receive data available 

; 	Read Register 1  Receive Condition Flag  
RX_EOF		EQU	80H	;  Receive End of frame 
RX_CRCYUK	EQU	40H	;  CRC/framing error flag 
RX_OVRN		EQU	20H	;  RX overrun error flag 
RX_PARTY	EQU	10H	;  RX parity error flag 
RX_RESID	EQU	0EH	;  Residual data code 
TX_ALSNT	EQU	01H	;  Transmit all-sent flag (async) 

; 	Read Register 2 holds the Interrupt Vector, if used 
; 	Channel A holds the base value of the vector, while 
; 	the Channel B value may include status modifications 

; 	Read Register 3.  Interrupt pending flags  
; 	Channel A only, Channel B is always zero  
A_RX_IP		EQU	20H	;  Channel A Rx interrupt pend. 
A_TX_IP		EQU	10H	;  Channel A Tx interrupt pend. 
A_EXT_IP	EQU	08H	;  Channel A External int. pend. 
B_RX_IP		EQU	04H	;  Channel B Rx interrupt pend. 
B_TX_IP		EQU	02H	;  Channel B Tx interrupt pend. 
B_EXT_IP	EQU	01H	;  Channel B External int. pend. 

; 	Read Register 8 is the data register, which is usually 
; 	accessed directly					

; 	Read Register 10.  Misc. status flags 
CLK1_MIS	EQU	80H	;  One clock missing (FM mode) 
CLK2_MIS	EQU	40H	;  Two clock missing (FM mode) 
LOOP_SND	EQU	10H	;  Loop sending (SDLC) 
ON_LOOP		EQU	02H	;  On loop flag (SDLC) 

; 	Read Register 12 is low half baud rate divider  
; 	Read Register 13 is high half baud rate divider 
; 	Read Register 15 shows what is in write register 15 

; **************************************************************
; 		SCC Write Registers				
; **************************************************************

; 	Write Register 0. Reg. pointers and Master Bits		

RST_EOM		EQU	0C0H	;  Reset Tx end-of-msg Flag 
RST_TCRC	EQU	80H	;  Reset Tx CRC generator 
RST_RCRC	EQU	40H	;  Reset RX CRC checker 
RST_IUS		EQU	38H	;  Reset highest Int. service 
RST_ERR1	EQU	30H	;  Reset error Flags in Reg 1 
RST_TXIP	EQU	28H	;  Reset Tx Interrupt pending 
NXT_RXEN	EQU	20H	;  Enable int. on next Rx char 
TX_ABORT	EQU	18H	;  Transmit an abort in SDLC 
RST_EXT		EQU	10H	;  Reset external/status interrupt 
PNT_HIGH	EQU	08H	;  Point high command 

; 	Write Register 1.  (Tx/Rx Interrupt and DMA)  

DMA_ENAB	EQU	80H	;  Wait/DMA request enable 
DMA_RQST	EQU	40H	;  WaitDMA request function 
DMA_TXRX	EQU	20H	;  Wait/DMA on Rx or TX 
RXI_SPCL	EQU	18H	;  Int on RX special cond only  
RXI_ALL		EQU	10H	;  Int on all Rx and special cond 
RXI_FRST	EQU	08H	;  Int on first RX and special cond 
PAR_SPCL	EQU	04H	;  Parity is special RX condition 
TXI_ENAB	EQU	02H	;  Tx Interrupt Master Enable 
EXI_ENAB	EQU	01H	;  External Interrupt Enable 

; 	Write Register 2 contains the Interrupt Vector   

; 	Write Register 3.  Rx parameters and control   

RX_8BITS	EQU	0C0H	;  RX 8 bits per character 
RX_6BITS	EQU	80H	;  RX 6 bits per character 
RX_7BITS	EQU	40H	;  RX 7 bits per character 
RX_5BITS	EQU	0H	;  RX 5 bits/char default = 00 
AUTOENAB	EQU	20H	;  Enable auto mode DCD and CTS 
HUNT_ON		EQU	10H	;  Turn on hunt mode in RX 
RX_CRCON	EQU	08H	;  Turn on RX crc checker 
ADSEARCH	EQU	04H	;  Turn on address search in SDLC 
SYNC_INH	EQU	02H	;  Sync load inhibit flag 
RX_ENABL	EQU	01H	;  Master RX enable flag 

; 	Write Register 4.  Tx/Rx misc parameters and modes 
X64_CLCK	EQU	0C0H	;  X64 clock mode 
X32_CLCK	EQU	80H	;  X32 clock mode 
X16_CLCK	EQU	40H	;  X16 clock mode 
X1_CLCK		EQU	0H	;  X1 clock mode default 00 
SYNC_EXT	EQU	30H	;  External sync mode 
SYNC_SDL	EQU	20H	;  SDLC sync mode 
SYNC_16B	EQU	10H	;  16 bit sync character 
SYNC_8B		EQU	0H	;  8 bit sync mode default 00 
STOP_2BT	EQU	0cH	;  2 stop bits in async mode 
STOP_15B	EQU	08H	;  1.5 stop bits in async mode 
STOP_1BT	EQU	04H	;  1 stop bit in async mode 
SYNC_MOD	EQU	0H	;  SYNC mode default by 00 
EVEN_PAR	EQU	02H	;  Parity mode even/odd* 
ENAB_PAR	EQU	01H	;  Enable parity check/gen. 

; 	Write Register 5.  TX parameters and commands 

DTR_PIN		EQU	80H	;  SCC DTR pin set 
TX_8BITS	EQU	60H	;  Tx 8 bits per character 
TX_6BITS	EQU	40H	;  Tx 6 bits per character 
TX_7BITS	EQU	20H	;  Tx 7 bits per character 
TX_5BITS	EQU	0H	;  Tx 5 bits/char default 00 
TX_BREAK	EQU	10H	;  Send a break character 
TX_ENABL	EQU	08H	;  master transmit enable 
TX_CRC16	EQU	04H	;  SDLCCRC16 CRC mode 
RTS_PIN		EQU	02H	;  SCC RTS pin set 
TX_CRCEN	EQU	01H	;  Transmit CRC enable 

; 	Write Register 6 is the address field in SDLC mode  
; 	Write Register 7 is loaded with sync byte, Flag in SDLC 
; 	Write Register 8 is Tx data register			

; 	Write Register 9.  Master Interrupt control register 
; 			   Same register in both channels.   

SCCRESET	EQU	0c0H	;  Master chip hardware reset 
CHARESET	EQU	80H	;  Channel A reset 
CHBRESET	EQU	40H	;  Channel B reset 
INT_HILO	EQU	10H	;  interrupt status *lo/hi  
INT_ENAB	EQU	08H	;  Master chip interrupt enable 
DIS_LOWR	EQU	04H	;  Disable lower interrupt chain 
NO_VECTR	EQU	02H	;  No interrupt vector from SCC 
ENAB_VIS	EQU	01H	;  enable vector-includes-status 

; 	Write Register 10.  Misc. Tx/Rx control bits  

CRC_1or0	EQU	80H	;  CRC preset to one or zero 
FM0_MODE	EQU	60H	;  set mode to FM0 
FM1_MODE	EQU	40H	;  set mode to FM1 
NRZIMODE	EQU	20H	;  set mode to NRZI 
NRZ_MODE	EQU	0H	;  set mode to NRZ 
ACT_POLL	EQU	10H	;  Got active on poll in SDLC loop 
IDL_MARK	EQU	08H	;  *Flag/Mark while Tx idle 
UND_ABRT	EQU	04H	;  *Flag/Abort on Tx underrun 
SDLCLOOP	EQU	02H	;  Set SDLC loop mode 
SYNC_8_6	EQU	01H	;  *8bit/6bit sync mode 

; 	Write Register 11.  Clock control register  
XTAL_OSC	EQU	80H	;  Use internal xtal oscillator 
RXC_DPLL	EQU	60H	;  Rx clock  from DPLL 
RXC_BRG		EQU	40H	;  Rx clock from baud rate generator 
RXC_TRXC	EQU	20H	;  Rx clock from TRXC pin 
RXC_RTXC	EQU	00H	;  Rx clock from RTXC pin 
TXC_DPLL	EQU	18H	;  Tx clock from DPLL out 
TXC_BRG		EQU	10H	;  Tx clock from baud rate generator 
TXC_TRXC	EQU	08H	;  Tx clock from TRXC pin 
TXC_RTXC	EQU	0H	;  Tx clock from RTXC pin 
TRXC_OUT	EQU	04H	;  TRXC pin is an *in/out 
TRXC_DPL	EQU	03H	;  TRXC pin out from DPLL 
TRXC_BRG	EQU	02H	;  TRXC pin out from BRG 
TRXC_TXC	EQU	01H	;  TRXC pin out from Tx clock 
TRXC_XTL	EQU	0H	;  TRXC pin out from XTAL OSC 

; 	Write Register 12 is the low half of the Baud Rate Generator 
; 	Write Register 13 is the hi half of the Baud Rate Generator 

; 	Write Register 14.  Misc mode and control bits   

MOD_NRZI	EQU	0E0H	;  Set operating mode to NRZI 
MOD_FM		EQU	0C0H	;  Set operating mode to FM  
DPLL_RTX	EQU	0A0H	;  Set DPLL source to RTXC pin 
DPLL_BRG	EQU	80H	;  Set DPLL source to baud rate gen 
DPLL_DIS	EQU	60H	;  Disable DPLL, force search mode 
RST_MCLK	EQU	40H	;  Reset missing clock latches 
DPLLSRCH	EQU	20H	;  DPLL enter search mode 
LCL_LOOP	EQU	10H	;  Local loop-back mode TxD to RxD 
AUTOECHO	EQU	08H	;  Auto-echo mode (RxD = TxD + RxD) 
DTR_FUNC	EQU	04H	;  *DTR/REQ DTR pin function 
BRG_CLCK	EQU	02H	;  *RTXC/SCC clock baud-rate-gen in 
ENAB_BRG	EQU	01H	;  Enable baud-rate-generator 

; 	Write Register 15.  External/status interrupt control 

RXIE_BRK	EQU	80H	;  Rx Break/abort detect Int. enable 
TXIE_EOM	EQU	40H	;  Tx EOM/underrun Int. enable 
CTS_IE		EQU	20H	;  CTS pin interrupt enable 
HUNT_IE		EQU	10H	;  SYNC/HUNT interrupt enable 
DCD_IE		EQU	08H	;  Carrier detect interrupt enable 
BRG0_IE		EQU	02H	;  Baud-rate-gen. zero count int. enab 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; EAGLE DATA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SIGN_ON	DB	'                 SYNCHRONUS PC PACKET PROGRAM'
	DB	0DH,0AH,'$'
;
SINT_MSG DB	'  SETTING UP THE INTERRUPT'
	DB	0DH,0AH,'$'
;
STRT_MSG DB	'  GOING TO INITIALIZATION ROUTINE'
	DB	0DH,0AH,'$'
;
INIT_MSG DB	'         8530 INITIALIZATION COMPLETE'
	DB	0DH,0AH,'$'
;
DELAY_MSG	DB	'DELAYING FOR TRANSMITTER TO COME UP OR CRC+FLAG'
	DB	0DH,0AH,'$'
;
DONE_MSG DB	'         8530 TESTING COMPLETE'
	DB	0DH,0AH,'$'
;
FRAME	DB	82H,94H,72H,0B0H,40H,40H,60H
;                A   J   9   X           0
	DB	82H,94H,72H,0B0H,40H,40H,60H
;                A   J   9   X           0
;
	DB	0AEH,66H,0ACH,88H,40H,40H,6BH,03H,0F0H
;		  W   3   V   D           5
;
	DB	'Testing IBM-PC w/Eagle HDLC controller card...'
	DB	0DH,0AH
	DB	'The quick brown fox jumped over the lazy dogs '
	DB	'back 0123456789 times.'
	DB	0DH,0AH
FRSIZ	EQU	$-FRAME
;
;
R1BCNT		DW	0	;RECEIVE BUFFER COUNT
IBUFFER		DB	327 DUP (0)	;327 BYTE INPUT BUFFER
EBUFFER		DB	0DH,'$'	;END OF RECEIVE BUFFER FOR DISPLAY
T1BSTA		DB	0	;LEVEL 1 CHAN B XMIT STATUS
R1BSTA		DB	0	;LEVEL 1 CHAN B RX STATUS
T1BCNT		DW	0	;CHAN B XMIT COUNT TO SEND
FPTR		DW	0	;FRAME POINTER
INTPEND		DB	0	;INT PENDING BYTE
STATREAD	DB	0	;EXT STATUS READ
DONEBIT		DB	0	;LAST ADDRESS TO DISP RX PROCESS
;
; NOTE IF IBUFPTR1 = IBUFPTR2 THEN INPUT BUFFER IS EMPTY
;
DATA	ENDS
;
; AT LAST, THE CODE SEGMENT, STANDARD PROGRAM PROLOGUE
;
CODE	SEGMENT	PARA PUBLIC 'CODE'
START	PROC	FAR	;TYPE FAR MEANS INTER-SEGMENT
	ASSUME	CS:CODE
;
; IN THE BEGINNING, THERE WAS PCDOS.  AFTER LOADING OUR
; PROGRAM, IT POINTED THE DS REGISTER AT OUR PROGRAM SEGMENT
; PREFIX (PSP).  IT NEEDS TO POINT AT OUR DATA SEGMENT
;
	PUSH	DS	;SAVE PSP SEGMENT ADDRESS
	MOV	AX,0	;PUSH OFFSET 0 ONTO STACK
	PUSH	AX	;SAVE RET ADDR OFFSET (PSP+0)
	MOV	AX,DATA ;GET DATA ADDRESS INTO A
	MOV	DS,AX	;THEN INTO DS
	ASSUME	DS:DATA ;AND ESTABLISH DATA SEGMENT
			;ADDRESSABILITY
;
; NOW SOME REAL CODE
;  FIRST, ANNOUNCE OUR MISERABLE PRESENCE
;
	MOV	DX,OFFSET SIGN_ON	;POINT AT MESSAGE
	MOV	AH,PRT_STR		;SET UP THE CALL
	INT	DOSINT			;DO THE DOSCALL
;
;  TURN OFF EAGLE INT
;
;	MOV	DX,ECTRL_REG		;POINT AT EAGLE CONTROL REG
;	MOV	AL,0			;INTS OFF
;	OUT	DX,AL			;SEND IT OUT	
;
;  NOW SET UP OUR INTERRUPT STRUCTURE
;  OUR 8530 HARDWARE INTERRUPT IS WIRED TO INTERRUPT 2
;  SO HERE WE REPLACE THE IBM PROVIDED INTERRUPT SERVICE
;  ROUTINE ADDRESS WITH OUR ROUTINES ADDRESS
;  THE TABLE ENTRY IS OBTAINED FROM THE IBM PC TECHNICAL MANUAL
;  PAGE 5-5
;
	MOV	DX,OFFSET SINT_MSG
	MOV	AH,PRT_STR
	INT	DOSINT
;
	CLI				;DISABLE ALL INTS WHILE WE MODIFY
	MOV	AX,0
	MOV	ES,AX			;POINT THE EXTRA SEGMENT
					;AT THE INTERRUPT SERVICE
					;ROUTINE ADDRESS TABLE
	MOV	DI,28H			;OFFSET FOR RESERVED IRQ2
	MOV	AX,OFFSET INT_8530	;OFFSET OF OUR ROUTINE
	CLD				;SET FORWARD STRING OPERATIONS
	STOSW				;STORE ONE WORD AT AT TIME
	MOV	AX,CS			;SEGMENT OF OUR ROUTINE
	STOSW				;STORE ONE WORD AT TIME
;
; INTERRUPT MASK REGISTER (IMR)...A 0 ENABLES THE INTERRUPT
;    PRINTER  FLOPPY  HARD   COM1   COM2  RESERVED   KBD   TIMER
;     IRQ7     IRQ6   IRQ5   IRQ4   IRQ3   IRQ2     IRQ1    IRQ0
;                                         EAGLE
;
	MOV	DX,0021H	;POINT AT IMR
	MOV	AL,0	;ENAB ALL INTS
	OUT	DX,AL	;WRITE INT MASK REGISTER
;
;  NOW INITIALIZE THE 8530 FOR PACKET WORK
;
	MOV	DX,OFFSET STRT_MSG
	MOV	AH,PRT_STR
	INT	DOSINT
;
	CALL	INIT_8530
;
;  SET UP TO TRANSMIT SOME FIRST
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG5
	OUT	DX,AL
	MOV	AL,DTR_PIN+TX_8BITS+TX_ENABL+TX_CRCEN
	OUT	DX,AL
;
;  INITIALIZE VARIABLES
;
	MOV	T1BSTA,0
	MOV	T1BCNT,FRSIZ
	MOV	FPTR,0
;
;  TURN ON TRANSMITTER
;
	CALL	T1BTON
WAITT:
	MOV	AL,T1BSTA
	CMP	AL,0	;DONE YET?
	JNE	WAITT
;
;  TURN OFF THE TRANSMITTER, GET READY FOR RX
;
	CALL	T1BOFF		;TURN IT OFF
;
;  NOW RECEIVE ONE FRAME FOR TESTING
;
	CALL	R1BRON		;TURN ON RECEIVER
WAITR:
	MOV	AL,R1BSTA	;GET RECEIVER STATUS
	CMP	AL,0		;IS RECEIVER STILL RECEIVING
	JNE	WAITR		;YES, RECEIVING
	CALL	R1BROFF		;TURN OFF RECEIVER,ALL DONE
;
;  DISPLAY WHAT WE GOT
;
	CALL	DISPRX		;DISPLAY FRAME RECEIVED
;
;  ALL DONE
;
QUIT:
	MOV	DX,OFFSET DONE_MSG	;POINT AT MESSAGE
	MOV	AH,PRT_STR		;SET UP THE CALL
	INT	DOSINT			;DO THE DOSCALL
	RET				;TO DOS
;
;  INT_8530
;  INTERRUPT SERVICE SUBROUTINE FOR 8530
;  HERE WE DISCOVER WHAT WOKE US UP
;  WAS IT A SEND, A RECEIVE, B SEND, B RECEIVE, ETC.
;
INT_8530	PROC	FAR
	PUSH	DS	;SAVE EVERYTHING
	PUSH	BX
	PUSH	AX
;
;  ESTABLISH DATA SEGMENT ADDRESSABILITY
;
	MOV	AX,DATA
	MOV	DS,AX
;
;  WHAT CAUSED THIS 8530?  DISCOVER REASON
;  WE ARE HERE AND NOT THERE
;  CHECKOUT THE INTERRUPT PENDING FLAGS
;  NOTE, CHANNEL A, NOT B....B IS ALWAYS 0
;
	MOV	DX,SCCA_CTL	;LOAD UP PORT ADDRESS
	MOV	AL,SCCREG3	;SELECT REG 3
	OUT	DX,AL		;SEND IT
	IN	AL,DX		;READ THE THING
	MOV	INTPEND,AL	;SAVE IT OFF
;
; TEST ALL THE POSSIBILITIES 
; AND DO SOMETHING, NOTICE AFTER ONE HIT, WE CHECK THE REST ALSO
;
CK1:
	MOV	AL,INTPEND	;GRAB INTERRUPT PENDING
	AND	AL,A_RX_IP	;MASK OFF JUST WHAT WE WANT
	CMP	AL,A_RX_IP	;TEST FOR IT
	JE	SCARDA		;WE GOT IT, GO PROCESS
;
CK2:
	MOV	AL,INTPEND
	AND	AL,A_TX_IP
	CMP	AL,A_TX_IP
	JE	SCATBE
;
CK3:
	MOV	AL,INTPEND
	AND	AL,A_EXT_IP
	CMP	AL,A_EXT_IP
	JE	SCAEXT
;
CK4:
	MOV	AL,INTPEND
	AND	AL,B_RX_IP
	CMP	AL,B_RX_IP
	JE	SCBRDA
;
CK5:
	MOV	AL,INTPEND
	AND	AL,B_TX_IP
	CMP	AL,B_TX_IP
	JE	SCBTBE
;
CK6:
	MOV	AL,INTPEND
	AND	AL,B_EXT_IP
	CMP	AL,B_EXT_IP
	JE	SCBEXT
;
;  LOST INTERRUPT
;
CK7:
	JMP	IDONE		;WE LOST SOMEHOW
;
;  HANDLER AREA.....ALL INTERRUPT HANDLERS GO BELOW HERE
;
;  RECEIVE INTERRUPT PENDING - CHANNEL A
;
SCARDA:
	JMP	CK2		;STUB
;
;  TRANSMIT INTERRUPT PENDING - CHANNEL A
;
SCATBE:
	JMP	CK3		;STUB
;
;  EXTERNAL INTERRUPT PENDING - CHANNEL A
;
SCAEXT:
	JMP	CK4		;STUB
;
;  CHANNEL B EXTERNAL STATUS INTERRUPT ROUTINE
;
SCBEXT:
	MOV	STATREAD,0	;START CLEAN
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG0	;POINT AT R0
	OUT	DX,AL		;SEND IT OUT
	IN	AL,DX		;READ THE THING
	MOV	STATREAD,AL	;SALT IT AWAY
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG0	;POINT AT R0
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,RST_EXT	;RESET EXT STATUS INT
	OUT	DX,AL		;SEND IT OUT
	JMP	CK7		;FINISHED
;
;  TRANSMIT INTERRUPT PENDING ROUTINE
;  HERE THE 8530 INDICATED TBE
;  HE WANTS ANOTHER CHARACTER TO GO
;  OUT.  THIS TAKES THE PLACE OF OUR OLD
;  POLLING STATUS LOOKING FOR TBE
;
SCBTBE:
	MOV	AX,T1BCNT	;GET AMOUNT TO SEND
	CMP	AX,0		;ARE WE DONE
	JNE	SENDIT
	MOV	AL,T1BSTA	;DEC THE FLAG
	DEC	AL
	MOV	T1BSTA,AL	;WHEN IT GETS ZERO-QUIT
	JMP	CK6
;
;  SEND OUT A BYTE
;
SENDIT:	MOV	BX,FPTR		;GRAB POINTER
	MOV	AL,[FRAME+BX]	;GRAB BYTE
	MOV	DX,SCCB_DATA	;LOAD UP PORT ADDR
	OUT	DX,AL		;SEND IT OUT MODEM
;
	INC	BX		;BUMP POINTER
	MOV	FPTR,BX		;STUFF IT
;
	MOV	BX,T1BCNT	;GET LENGTH
	DEC	BX		;DROP IT BY ONE
	MOV	T1BCNT,BX	;RE-STUFF
	JMP	CK6		;QUIT THIS INTERRUPT
;
;  CHANNEL B RECEIVE DATA AVAILABLE INTERRUPT ROUTINE
;  THERE IS A CHARACTER COMING IN
;
SCBRDA:
;
;  FIRST CHECK FOR CHARACTER READY
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG0	;POINT AT R0
	OUT	DX,AL		;SEND IT OUT
	IN	AL,DX		;READ R0
	AND	AL,RX_RDA	;MASK IT
	CMP	AL,RX_RDA	;CHECK IT
	JNE	NOCHAR		;IF NO CHAR,MORE CHECKS
;
; HERE WE GOT A CHARACTER TO STUFF AWAY
;
	MOV	DX,SCCB_DATA	;LOAD UP PORT ADDR
	IN	AL,DX		;GRAB THE CHARACTER
;
	MOV	BX,R1BCNT	;GET LENGTH OF CURRENT BUFFER
	INC	BX		;BUMP THE POINTER
	MOV	R1BCNT,BX	;STORE IT AWAY
;
	MOV	[IBUFFER+BX],AL	;STUFF THE CHARACTER
	CMP	BX,327		;OVERRUN OUR BUFFER?
	JGE	QUITOVER	;DUMP ALL IF OVERRUN
;
;  HERE NO CHARACTER WAS WAITING
;  OR WE JUST GOT DONE PROCESSING ONE
;  SO FIND OUT IF WE ARE DONE DONE
;
NOCHAR:
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG1	;POINT AT R1
	OUT	DX,AL		;SEND IT OUT
	IN	AL,DX		;READ R1
	AND	AL,RX_EOF	;MASK OFF
	CMP	AL,RX_EOF	;DONE FRAME?
	JNE	RQUIT		;IF NOT, OK
;
;  CHECK FOR CRCOKFINEGOOD
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG1	;POINT AT R1
	OUT	DX,AL		;SEND IT OUT
	IN	AL,DX		;READ R1
	AND	AL,RX_CRCYUK	;MASK
	CMP	AL,RX_CRCYUK	;CRC FINE?
	JE	CRCBAD		;IF NOT,SHUCKS
CRCGUD:
	MOV	AL,'G'		;SHOW GOODGOOD
	CALL	DISPCHAR
	JMP	QUITOVER
CRCBAD:
	MOV	AL,'N'		;SHOW YUKYUK
	CALL	DISPCHAR
	JMP	QUITOVER
;
; CLEAR OUT ANY BOGUS ERRORS
; OR ERRORS I AM TOO LAZY TO FOOL WITH
;
RQUIT:
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG0	;POINT AT R0
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,RST_ERR1	;RESET ERRORS
	OUT	DX,AL		;SEND IT OUT
	JMP	CK5		;OK FOR WHATEVER NEXT
;
QUITOVER:
	MOV	BX,R1BCNT	;LOADUP LENGTH
	CMP	BX,7		;LENGTH LESS THAN ONE ADDRESS?
	JG	RECGUD		;IF NOT 0,OK FINE
;
; HERE PROCESS BAD FRAME
; 8530 THOUGHT HE RECEIVED A PACKET, BUT HE DIDN'T
;
	JMP	CK5
RECGUD:
	MOV	R1BSTA,0	;SHOW MAIN THAT WE ARE DONE
	JMP	CK5
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; INTERRUPT WRAPUP ENTRY POINT
; ALL INTERRUPTS FINISH HERE
; GET 8530 READY TO TRANSMIT OR RECEIVE MORE
;
IDONE: 
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDRESS
	MOV	AL,SCCREG0	;POINT AT R0
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,RST_IUS	;RESET HIGHEST INT UNDER SER
	OUT	DX,AL
;
;  INDICATE END OF INTERRUPT SERVICE TO 8259
;
	MOV	AL,20H		;EOI COMMAND
	OUT	20H,AL		;SEND IT TO 8259 INT CHIP
	POP	AX		;RESTORE
	POP	BX
	POP	DS
	IRET			;RETURN FROM INTERRUPT
INT_8530	ENDP		;END THE FAR PROCEDURE 
	
;
;  DISPCHAR
;  SUBROUTINE TO DISPLAY ONE CHARACTER ON SCREEN
;  ENTER WITH AL=CHARACTER TO BE DISPLAYED
;  USES VIDEO INTERFACE IN BIOS
;
DISPCHAR	PROC	NEAR
	PUSH	BX	;SAVEOFF BX
	MOV	BX,0	;SELECT DISPLAY PAGE 0
	MOV	AH,14	;FUNCTION CODE FOR WRITE
	INT	10H	;CALL VIDEO DRIVER IN BIOS
	POP	BX	;RESTORE BX REG
	RET		;TO CALLER
DISPCHAR	ENDP	;END THE NEAR PROCEDURE
;
;  R1BRON
;  SUBROUTINE TO TURN ON CHANNEL B RECEIVER FOR GRABBING FRAME
;
R1BRON	PROC	NEAR
;
;  SET UP 8530 TO RECEIVE
;  SET THE RECEIVE SPEED...32X CLOCK FOR DPLL REQUIRES
;  NEW SPEED SETTING TO MAINTAIN 1X FOR TRANSMITTING
;  REGISTER 12 AND 13 GET IT
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG12	;POINT AT R12
	OUT	DX,AL		;SEND IT
	MOV	AL,02EH		;1200BAUD /32 FOR RX LOW BYTE
	OUT	DX,AL		;SEND IT
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG13	;POINT AT R13
	OUT	DX,AL		;SEND IT
	MOV	AL,0		;1200BAUD /32 FOR RX HIGH BYTE
	OUT	DX,AL		;SEND IT
;
;  REGISTER 14 HAS A LOT GOING ON
;
;
;****************     Dave,
;		this is the area that needs to be tweaked
;		before the receive will work....
;			I am forcing an assembler error here
;		so you will find it fer surr.....
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG14	;POINT AT R14
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,MOD_NRZI	;SET NRZI MODE
	OUT	DX,AL
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG14	;POINT AT R14
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,DPLL_BRG+BRG_CLCK	;SET SOURCE=BRG
	OUT	DX,AL
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG14	;POINT AT R14
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,DPLLSRCH+BRG_CLCK	;DPLL ENTER SEARCH MODE
	OUT	DX,AL
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG14	;POINT AT R14
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,ENAB_BRG+BRG_CLCK	;ENABLE BRG
	OUT	DX,AL		;SEND IT OUT

;******************* end of area to correct....

;
;  REGISTER 15 TO PICK UP OUR DCD MAINLY
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG15	;POINT AT R15
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,RXIE_BRK+DCD_IE
	OUT	DX,AL		;SEND IT OUT
;
;  REGISTER 3 FOR RECEIVE PARAMETERS AND CONTROL
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG3	;POINT AT R3
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,RX_8BITS+RX_CRCON+RX_ENABL
	OUT	DX,AL		;SEND IT OUT
;
;  REGISTER 5 FOR DTR FLAME ON
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG5	;POINT AT R5
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,DTR_PIN	;RAISE DTR
	OUT	DX,AL		;SEND IT OUT
;
;  REGISTER 1 TO GET OUR RECEIVE INTERRUPTS GOING
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG1	;POINT AT R1
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,RXI_ALL+EXI_ENAB
	OUT	DX,AL		;SEND IT OUT
;
; WRAPUP
;
	STI			;INTS ON IBM
;
;  ENABLE ON EAGLE BOARD 
;
	MOV	DX,ECTRL_REG
	MOV	AL,08H		;ENABLE INTS
	OUT	DX,AL
;
	MOV	R1BSTA,1	;SHOW RX GOINT ON
	MOV	R1BCNT,0	;ZERO OUT RECEIVE COUNT
	RET			;ALL DONE
R1BRON	ENDP
;
;  R1BROFF
;  SUBROUTINE TO TURN OFF CHANNEL B RECEIVER
;
R1BROFF	PROC	NEAR
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG3	;POINT AT R3
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0		;TURN OFF RX,RX CRC GEN
	OUT	DX,AL		;SEND IT OUT
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG15	;POINT AT R15
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0		;TURN OFF RX EXT INTS
	OUT	DX,AL		;SEND IT OUT
;
	MOV	R1BSTA,0	;SHOW RX OFF
;
;  TURN OFF INTS TO EAGLE BOARD
;
	MOV	DX,ECTRL_REG	;POINT AT EAGLE CONTROL REG
	MOV	AL,0		;ALL OFF
	OUT	DX,AL		;SEND IT OUT
;
	RET		;ALL DONE
R1BROFF	ENDP
;
;  T1BTON
;  SUBROUTINE TO TURN ON CHANNEL B TRANSMITTER AND SEND FRAME
;
T1BTON	PROC	NEAR
;
;  SHUT OFF RECEIVER
;
	CALL	R1BROFF		;SHUT DOWN RECEIVING
;
;  TURN ON RTS AND TRANSMIT ENABLE
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDRESS
	MOV	AL,SCCREG5	;POINT AT R5
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0EBH		;RTS,TXEN,TX8,TXCRCEN,DTR
	OUT	DX,AL		;SEND IT OUT
;
;  DELAY FOR XMITTER TO CLACK RELAYS ON
;
	CALL	DELAY
;
;  RESET TRANSMITTER CRC GENERATOR
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDRESS
	MOV	AL,SCCREG0	;POINT AT R0
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,80H		;CRC ON
	OUT	DX,AL		;SEND IT OUT
;
;  SEND OUT FIRST BYTE TO PRIME THE PUMP
;  NOTE AT THIS POINT, TX INTS ARE OFF
;
	MOV	BX,FPTR		;GRAB POINTER
	MOV	AL,[FRAME+BX]	;GRAB BYTE
	MOV	DX,SCCB_DATA	;LOAD UP PORT ADDR
	OUT	DX,AL		;SEND IT OUT
	INC	BX		;BUMP POINTER
	MOV	FPTR,BX		;STUFF IT
	MOV	BX,T1BCNT	;GET PACKET LENGTH
	DEC	BX		;DROP IT BY ONE
	MOV	T1BCNT,BX	;SAVE IT BACK 
;
;  RESET TRANSMIT UNDERRUN/END OF MESSAGE
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG0	;POINT AT R0
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0C0H		;TXUNDERRUN/EOM
	OUT	DX,AL		;SEND IT OUT
;
;  TURN ON TRANSMIT INTERRUPT ENAB
;    AND EXTERNAL INTERRUPT ENABLE
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG1	;POINT AT R1
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,TXI_ENAB+EXI_ENAB	;TXINT AND EXIINT
	OUT	DX,AL		;SEND IT OUT
;
;  TURN ON TRANSMIT UNDERRUN/EOM INTERRUPT ENABLE
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDRESS
	MOV	AL,SCCREG15	;POINT AT R15
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,40H		;TXUNDERRUN/EOM INTENAB
	OUT	DX,AL		;SEND IT OUT	 
;
;  WRAPUP
;	
	MOV	T1BSTA,1	;SHOW XMIT GOING ON
;
	STI			;INTS ON IBM
;
;  ENABLE ON EAGLE BOARD
;
	MOV	DX,ECTRL_REG
	MOV	AL,08H		;ENABLE INTS
	OUT	DX,AL
;
	RET
T1BTON	ENDP
;
;  T1BOFF
;  SUBROUTINE TO TURN OFF CHANNEL B TRANSMITTER
;
T1BOFF	PROC	NEAR
;
;  TURN OFF TRANSMIT INTERRUPTS
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG1	;POINT AT R1
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0		;TXINT OFF
	OUT	DX,AL		;SEND IT OUT
;
;  DELAY SO ENDING FLAG CAN FLY OUT THE DOOR
;
	CALL	DELAY		;WAIT FOR CRC&FLAG
;
;  TURN OFF RF AND HDLC TRANSMITTER
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG5	;POINT AT R5
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0		;RTS OFF,TXEN OFF
	OUT	DX,AL		;SEND IT OUT
;
;  TURN OFF TRANSMITTER UNDERRUN/END OF MESSAGE INT
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDR
	MOV	AL,SCCREG15	;POINT AT R15
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0		;TXUNDERRUN/EOM OFF
	OUT	DX,AL		;SEND IT OUT
;
	MOV	T1BSTA,0	;SHOW XMIT OFF
;
;  TURN OFF INTS ON EAGLE BOARD
;
	MOV	DX,ECTRL_REG	;POINT AT EAGLE CONTROL REG
	MOV	AL,0		;ALL OFF
	OUT	DX,AL		;SEND IT OUT
;
	RET
T1BOFF	ENDP
;
;  DELAY
;  SUBROUTINE TO DELAY SO SOMETHING CAN HAPPEN
;
DELAY	PROC	NEAR
	MOV	DX,OFFSET DELAY_MSG	;POINT AT MESSAGE
	MOV	AH,PRT_STR	;SET UP THE CALL
	INT	DOSINT
	MOV	DX,OFFSET DELAY_MSG
	MOV	AH,PRT_STR
	INT	DOSINT
	RET
DELAY	ENDP
;	
;  INIT_8530
;  SUBROUTINE TO RESET THE 8530 SCC
;
INIT_8530	PROC	NEAR
;
; MASTER RESET THE 8530 TO PREPARE IT FOR REAL WORK
; USE WR9, THE MASTER INTERRUPT CONTROL REG
;
	MOV	DX,SCCB_CTL	;LOAD UP PORT ADDRESS
	MOV	AL,SCCREG9	;POINT AT WR9
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,SCCRESET	;FORCE HARDWARE RESET BITS
	OUT	DX,AL		;SEND IT OUT
;
	XCHG	AX,BX		;SWAP TIME WASTER
	XCHG	AX,BX		;SWAP-EM BACK TIME WASTER
	XCHG	AX,BX		;SWAP TIME WASTER
	XCHG	AX,BX		;SWAP-EM BACK TIME WASTER
;
; SET UP THE 8530 BAUD RATE GENERATOR, BEGIN WITH DIV
; USE REGISTER 4, THE TX/RX MISC PARMS AND MODES REG
	MOV	AL,SCCREG4	;POINT AT R4
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,20H		;X1 CLOCK
				;SDLC MODE
	OUT	DX,AL		;SEND IT OUT
;
; TURN OFF INTERRUPTS
;
	MOV	AL,SCCREG1	;POINT AT R1
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0		;ALL OFF
	OUT	DX,AL		;SEND IT OUT
;
; TURN OFF EXTERNAL INTERRUPTS (LIKE CTS/CD)
;
	MOV	AL,SCCREG15	;POINT AT R15
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,SCCNULL	;TURN OFF EXT INTS
	OUT	DX,AL		;SEND IT OUT
;
;  SET UP THE RECEIVER
;
	MOV	AL,SCCREG3	;POINT AT R3
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0C0H		;8 BITS/CHARACTER
				;ALL OFF
	OUT	DX,AL		;SEND IT OUT
;
;  SET UP THE TRANSMITTER
;
	MOV	AL,SCCREG5	;POINT AT R5
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,69H		;TX8,TXEN,TXCRCEN
				;ALL OFF
	OUT	DX,AL		;SEND IT OUT
;
;  NULL OUT ADDRESS FIELD FOR SDLC TO START
;
	MOV	AL,SCCREG6	;POINT AT R6
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,SCCNULL	;NO ADDRESS
	OUT	DX,AL		;SEND IT OUT
;
;  SDLC FLAG SET
;
	MOV	AL,SCCREG7	;POINT AT R7
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,7EH		;01111110
	OUT	DX,AL		;SEND IT OUT
;
;  TURN OFF ALL INTERRUPTS WITH MASTER INTERRUPT
;  CONTROL REGISTER
;
	MOV	AL,SCCREG9	;POINT AT R9
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0		;TURN OFF
	OUT	DX,AL		;SEND IT OUT
;
; WE WANT CRC AND NRZI
;
	MOV	AL,SCCREG10	;POINT AT R10
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0A0H		;CRC ON
				;SDLC PLEASE
	OUT	DX,AL		;SEND IT OUT
;
; SET UP THE CLOCK CONTROL REGISTER
; EAGLE HAS CRYSTAL OSCILLATOR GOING INTO PCLK (PIN 20)
; TRXCB (PIN 26) IS OUTPUT TO RTXCB (PIN 28) INPUT
; TRXCA (PIN 14) IS OUTPUT TO RTXCA (PIN 12) INPUT
;
	MOV	AL,SCCREG11	;POINT AT R11
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,70H		;NO XTAL
				;RX CLOCK=DPLL
				;TX CLOCK=BRG
				;TRXC=INPUT
	OUT	DX,AL		;SEND IT OUT
;  
;  SET UP THE BAUD RATE GENERATOR,1200 BAUD
;  TIMES ONE CLOCK TO BEGIN, 1534 DECIMAL BY THE FORMULA
;  05FE HEX
;
	MOV	AL,SCCREG12	;POINT AT R12
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,0FEH		;LOW PART OF BAUD RATE
	OUT	DX,AL		;SEND IT OUT
;
	MOV	AL,SCCREG13	;POINT AT R13
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,05		;HIGH PART OF BRG
	OUT	DX,AL		;SEND IT OUT
;
;  TURN ON THE BAUD RATE GENERTOR NOW IT IS SET
;
	MOV	AL,SCCREG14	;POINT AT R14
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,ENAB_BRG+BRG_CLCK	;TURN IT ON
	OUT	DX,AL		;SEND IT OUT
;
;  SET NRZI MODE
;
	MOV	AL,SCCREG14	;POINT AT R14
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,MOD_NRZI+ENAB_BRG+BRG_CLCK	;SELECT NRZI
	OUT	DX,AL		;SEND IT OUT
;
;  SET SOURCE TO BAUD RATE GENERATOR
;
	MOV	AL,SCCREG14	;POINT AT R14
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,DPLL_BRG+ENAB_BRG+BRG_CLCK	;SET SOURCE TO BRG
	OUT	DX,AL		;SEND IT OUT
;
;  DPLL ENTER SEARCH MODE
;
	MOV	AL,SCCREG14	;POINT AT R14
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,DPLLSRCH+ENAB_BRG+BRG_CLCK	;ENTER SEARCH
	OUT	DX,AL		;SEND IT OUT
;
;  ENABLE INTERRUPTS, INCLUDING STATUS,
;   AND NO VECTORS (VICTOR)
;
	MOV	AL,SCCREG9	;POINT AT R9
	OUT	DX,AL		;SEND IT OUT
	MOV	AL,INT_ENAB+ENAB_VIS+NO_VECTR	;MASTER CHIP INT ENAB
	OUT	DX,AL		;SEND IT OUT
;
;  NOW, ANNOUNCE PARTIAL VICTORY
;
	MOV	DX,OFFSET INIT_MSG	;POINT AT MESSAGE
	MOV	AH,PRT_STR		;SET UP THE CALL
	INT	DOSINT			;DO THE DOSCALL
;
;  GO BACK
;
	RET				;FROM 8530 INIT
INIT_8530	ENDP			;END THE NEAR PROC
;
;  DISPRX
; SUBROUTINE TO DISPLAY CONTENTS OF RECEIVE BUFFER 'IBUFFER'
;
DISPRX	PROC	NEAR
	CALL	CALLSIGN	;FIXUP CALLSIGN FOR DISPLAY
;
; BEGIN THE DISPLAY
;
	MOV	AL,0DH
	CALL	DISPCHAR
	MOV	AL,0AH
	CALL	DISPCHAR
;
	MOV	BX,0		;INIT LOOP COUNTER
DISPM:
	MOV	AL,[IBUFFER+BX]	;LOAD UP THE CHARACTER
;
;  BOUND THE CHARACTER TO ASCII DISPLAYABLE
;
	CMP	AL,20H
	JL	FIX		;IF TOO LITTLE, PRINT .
	CMP	AL,07FH
	JG	FIX		;IF TOO GREAT, PRINT .
	CALL	DISPCHAR
	JMP	MOVEON		;IF OK, KEEP GOINT
FIX:
	MOV	AL,2EH		;STUFF A PERIOD
	CALL	DISPCHAR
MOVEON:
	INC	BX
	CMP	BX,327		;DONE TOTALLY BARF OUT?
	JNZ	DISPM
;
; FINISH UP THE DISPLAY
;
DONEFR:
	MOV	AL,0DH		;CR
	CALL	DISPCHAR
	MOV	AL,0AH		;LF
	CALL	DISPCHAR
;
; WIPE OUT THE RECEIVE BUFFER WITH 00
;
	MOV	BX,0
WIPE:
	MOV	[IBUFFER+BX],0	;STUFF A ZERO
	INC	BX
	CMP	BX,327
	JNZ	WIPE
;
; DONE DISPLAY
;
	RET
DISPRX	ENDP
;
;  SUBROUTINE TO FIXUP CALLSIGNS
;
CALLSIGN	PROC	NEAR
	MOV	BX,0	;BEGIN POINTER AT ZERO
CALLM:
	CALL	DO7
	CMP	DONEBIT,1	;FINISHED?
	JZ	QCALL		;IF SO,QUIT
	JMP	CALLM		;ELSE KEEP GOING
QCALL:
	MOV	DONEBIT,0	;RESET IT
	RET
CALLSIGN	ENDP
;
; SUBROUTINE TO DO 7 CHARACTERS OF CALLSIGN LIKE K8MMO 0
;
DO7	PROC	NEAR
	MOV	CX,0		;BEGIN BYTE COUNTER
DO7M:
	MOV	AL,[IBUFFER+BX]	;GRAB BYTE
	AND	AL,1		;MASK LSB
	CMP	AL,1		;LAST BYTE OF LAST ADDR?
	JZ	SETLAST		;IF SO FLAG MAIN SUBROUTINE
RBYTE:
	MOV	AL,[IBUFFER+BX]	;GRAB BYTE AGAIN
	ROR	AL,1		;SHIFT RIGHT ONCE
	MOV	[IBUFFER+BX],AL	;STUFF SHIFTED BYTE
	INC	BX		;BUMP ADDRESS POINTER
	INC	CX		;BUMP COUNTER
	CMP	CX,7		;IF SEVEN,DONE
	JNZ	DO7M		;IF NOT DONE,KEEP IT UP
	RET
SETLAST:
	MOV	DONEBIT,1	;SET FLAG FOR DONENESS
	JMP	RBYTE		;NOW PROCESS LAST BYTE
DO7	ENDP	
;
;  END OF THE WHOLE PROGRAM AND SUBROUTINES
;
START	ENDP		;END THE FAR PROC
CODE	ENDS		;END THE CODE SEGMENT
	END	START	;STOP ASSEMBLING AND ENTRY IS 'START'
